import { ChartService } from '../../services/chart.service';
import { ChartDataService } from '../../services/chart-data.service';
import { ElementStrategyFactory, SingleLineSeries } from './chart.strategy';
import { SeriesStyle } from '../../interfaces/SeriesStyle';
import { SeriesType } from '../../interfaces/SeriesType';
export class Chart {
    constructor() {
        this.service = new SingleLineSeries();
    }
    dataSourceHandler(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.debounceCreation();
        }
    }
    componentWillLoad() {
        this.axisConfigs = {
            x: this.getDefaultXAxisConfig(),
            y: this.getDefaultYAxisConfig()
        };
    }
    componentDidLoad() {
        this.debounceCreation();
    }
    async onColumnsLoaded() {
        this.columns = await this.loadColumns();
        this.debounceCreation();
    }
    async onLinesLoaded() {
        this.lines = await this.loadLines();
        this.debounceCreation();
    }
    async onStackedColumnsLoaded() {
        this.stackedColumns = await this.loadStackedColumns();
        this.debounceCreation();
    }
    async onXAxisLoaded({ detail }) {
        this.xAxisElement = await this.loadXAxis();
        this.axisConfigs.x = Object.assign({}, this.axisConfigs.x, detail);
        this.debounceCreation();
    }
    async onYAxisLoaded({ detail }) {
        this.yAxisElement = await this.loadYAxis();
        this.axisConfigs.y = Object.assign({}, this.axisConfigs.y, detail);
        this.debounceCreation();
    }
    handleResize() {
        this.debounceCreation();
    }
    anjHoverHandler(svg, data, x) {
        const targetData = this.service.getTargetingData(data);
        const dataIndex = ChartService.getIndexOfClosestPoint(x, targetData);
        let activeData;
        if (this.activeIndex !== dataIndex) {
            this.activeIndex = dataIndex;
            const seriesStyle = this.service.getSeriesStyle();
            activeData = { data, axisConfigs: this.axisConfigs, activeIndex: this.activeIndex, seriesStyle, seriesMeta: this.seriesMeta };
            this.anjHover.emit(activeData);
            this.popoverTimer = window.setTimeout(() => {
                const popoverEl = this.service.getPopoverEl(svg, this.activeIndex);
                this.display(popoverEl);
                ChartService.showFocus(svg, this.activeIndex);
            }, Chart.popoverDebounceLength);
        }
    }
    addLineEvents(svg, chartOverlay, x, data) {
        chartOverlay
            .on('mousemove', () => this.anjHoverHandler(svg, data, x))
            .on('mouseout', () => this.anjEndHoverHandler(svg));
    }
    addColumnEvents(svgBars, data) {
        svgBars
            .on('mousemove', (_d, i, bars) => this.anjColumnHoverHandler(bars[i], data, i))
            .on('mouseout', () => this.anjEndHoverHandler(svgBars));
    }
    anjEndHoverHandler(svg) {
        switch (this.primarySeries) {
            case SeriesType.Line:
                window.clearTimeout(this.popoverTimer);
                this.activeIndex = null;
                this.popover.close();
                ChartService.hideFocus(svg);
            default:
                this.popoverTimer = window.setTimeout(() => {
                    this.activeIndex = null;
                    this.popover.close();
                    ChartService.hideFocus(svg);
                }, Chart.popoverDebounceLength);
        }
    }
    anjColumnHoverHandler(targetEl, data, index) {
        window.clearTimeout(this.popoverTimer);
        let activeData;
        activeData = { data, axisConfigs: this.axisConfigs, activeIndex: index, seriesStyle: SeriesStyle.Single, seriesMeta: this.seriesMeta };
        this.anjHover.emit(activeData);
        this.popoverEl = targetEl;
        this.display(this.popoverEl);
    }
    anjStackedHoverHandler(svg, data, dataIndex) {
        window.clearTimeout(this.popoverTimer);
        let activeData;
        if (this.activeIndex !== dataIndex) {
            this.activeIndex = dataIndex;
            const seriesStyle = this.service.getSeriesStyle();
            activeData = { data, axisConfigs: this.axisConfigs, activeIndex: this.activeIndex, seriesStyle, seriesMeta: this.seriesMeta };
            this.anjHover.emit(activeData);
            this.popoverEl = this.service.getPopoverEl(svg, this.activeIndex);
            this.display(this.popoverEl);
            ChartService.showFocus(svg, this.activeIndex);
        }
    }
    addStackedEvents(svg, bars) {
        bars
            .on('mousemove', (_d, i) => this.anjStackedHoverHandler(svg, this.data, i))
            .on('mouseout', () => this.anjEndHoverHandler(svg));
    }
    appendSVG(svg) {
        while (this.chartContainer.children.length > 1) {
            this.chartContainer.removeChild(this.chartContainer.lastChild);
        }
        this.chartContainer.appendChild(svg.node());
    }
    debounceCreation() {
        window.clearTimeout(this.creationTimer);
        this.creationTimer = window.setTimeout(async () => {
            this.chartSVG = await this.create();
            console.log(this.chartSVG);
        }, Chart.creationDebounceLength);
    }
    async display(target) {
        const popper = await this.popover.popper();
        if (popper) {
            popper.reference = target;
            popper.scheduleUpdate();
        }
        else {
            this.popover.open(target);
        }
    }
    async create() {
        let minMax;
        let stackedData;
        this.primarySeries = (this.stackedColumns) ? SeriesType.Stacked : (this.lines && !this.columns) ? SeriesType.Line : SeriesType.Column;
        this.serviceFactory = ElementStrategyFactory.get(this.primarySeries);
        if (this.primarySeries === SeriesType.Line) {
            this.axisConfigs.x.labelType = 'date';
        }
        this.data = this.handleDataSource();
        const dims = this.getDims();
        const flattenedData = this.service.flatten(this.data);
        if (this.stackedColumns) {
            stackedData = ChartService.formatDataAsStacked(this.data);
            const flattenedPointValues = stackedData.flat().flat();
            minMax = { min: Math.min(...flattenedPointValues), max: Math.max(...flattenedPointValues) };
            console.log('>>', stackedData);
        }
        const svg = ChartService.createSVG(dims);
        const x = (this.xAxisElement)
            ? await this.hydrateXAxis(dims, flattenedData, svg, this.primarySeries)
            : ChartService.addXAxis(svg, dims, flattenedData, this.axisConfigs, undefined, this.primarySeries);
        const y = (this.yAxisElement)
            ? await this.hydrateYAxis(dims, flattenedData, svg, minMax)
            : ChartService.addYAxis(svg, dims, flattenedData, this.axisConfigs, minMax);
        if (this.columns) {
            const x1 = ChartService.getX1(x, this.data, this.multiSeries);
            await this.hydrateColumns(svg, this.data, x, x1, y, dims);
        }
        if (this.stackedColumns) {
            await this.hydrateStackedColumns(svg, stackedData, x, y, dims);
        }
        if (this.lines) {
            await this.hydrateLines(svg, this.data, x, y, dims);
            if (this.primarySeries === SeriesType.Line) {
                const chartOverlay = ChartService.addOverlay(svg, dims);
                this.addLineEvents(svg, chartOverlay, x, this.data);
                ChartService.addRevealAnimation(svg, dims, this.axisConfigs);
            }
        }
        this.appendSVG(svg);
        return Promise.resolve(svg);
    }
    getDefaultXAxisConfig() {
        return {
            showAxis: false,
            labelType: 'string',
        };
    }
    getDefaultYAxisConfig() {
        return {
            showAxis: false,
            labelType: 'number',
        };
    }
    getDims() {
        const margin = { top: 20, right: 20, bottom: 20, left: 20 };
        const { height, width } = this.chartContainer.getBoundingClientRect();
        return {
            height: height - margin.top - margin.bottom,
            width: width - margin.left - margin.right,
            margin
        };
    }
    handleDataSource() {
        let formattedData;
        if (Array.isArray(this.dataSource[0])) {
            this.service = this.serviceFactory.get(SeriesStyle.Multi);
            formattedData = this.dataSource.map(d => ChartDataService.formatData(this.axisConfigs, d));
        }
        else {
            this.service = this.serviceFactory.get(SeriesStyle.Single);
            formattedData = ChartDataService.formatData(this.axisConfigs, this.dataSource);
        }
        this.multiSeries = this.service.isMultiSeries();
        this.popover.placement = this.service.getPopoverPlacement();
        return formattedData;
    }
    async hydrateColumns(svg, data, x, x1, y, dims) {
        this.seriesMeta = [];
        this.columns.forEach(async (column, i) => {
            const index = (!isNaN(column.dataIndex)) ? column.dataIndex : i;
            const columnData = (this.service.isMultiSeries()) ? data[index] : data;
            const bars = await column.init(svg, x, x1, y, columnData, index, dims);
            this.seriesMeta[index] = { name: column.name, netNegative: column.netNegative, color: undefined };
            this.addColumnEvents(bars, columnData);
        });
    }
    async hydrateLines(svg, data, x, y, dims) {
        this.seriesMeta = [];
        this.lines.forEach(async (line, i) => {
            const index = (!isNaN(line.dataIndex)) ? line.dataIndex : i;
            const lineData = (this.service.isMultiSeries()) ? data[index] : data;
            const color = await line.init(svg, x, y, lineData, index, dims, this.service.isMultiSeries());
            this.seriesMeta[index] = { name: line.name, color, netNegative: line.netNegative };
        });
    }
    async hydrateStackedColumns(svg, data, x, y, dims) {
        this.seriesMeta = [];
        this.stackedColumns.forEach(async (column, i) => {
            const index = (!isNaN(column.dataIndex)) ? column.dataIndex : i;
            const columnData = (this.service.isMultiSeries()) ? data[index] : data;
            const { bars, color } = await column.init(svg, x, y, columnData, index, dims);
            this.seriesMeta[index] = { name: column.name, netNegative: column.netNegative, color };
            this.addStackedEvents(svg, bars);
        });
    }
    hydrateXAxis(dims, data, svg, primarySeries) {
        return this.xAxisElement.init(svg, dims, data, this.axisConfigs, undefined, primarySeries);
    }
    hydrateYAxis(dims, data, svg, minMax) {
        return this.yAxisElement.init(svg, dims, data, this.axisConfigs, minMax);
    }
    loadColumns() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-column-series')).map(col => col.componentOnReady()));
    }
    loadLines() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-line-series')).map(col => col.componentOnReady()));
    }
    loadStackedColumns() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-stacked-column-series')).map(col => col.componentOnReady()));
    }
    loadXAxis() {
        return Promise.resolve(this.chartContainer.querySelector('anj-axis[dimension="x"]').componentOnReady());
    }
    loadYAxis() {
        return Promise.resolve(this.chartContainer.querySelector('anj-axis[dimension="y"]').componentOnReady());
    }
    hostData() {
        return {
            class: {
                'anj-rm-pop-arrow': !this.service.arrow()
            }
        };
    }
    render() {
        return [
            h("anj-overlay", { class: "popover", placement: "top", offset: "0, 15", ref: r => this.popover = r },
                h("anj-arrow", null),
                h("slot", null))
        ];
    }
    static get is() { return "anj-chart"; }
    static get properties() { return {
        "chartContainer": {
            "elementRef": true
        },
        "dataSource": {
            "type": "Any",
            "attr": "data-source",
            "mutable": true,
            "watchCallbacks": ["dataSourceHandler"]
        },
        "multiSeries": {
            "state": true
        }
    }; }
    static get events() { return [{
            "name": "anjHover",
            "method": "anjHover",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get listeners() { return [{
            "name": "anjColumnDidLoad",
            "method": "onColumnsLoaded"
        }, {
            "name": "anjLineDidLoad",
            "method": "onLinesLoaded"
        }, {
            "name": "anjStackedColumnDidLoad",
            "method": "onStackedColumnsLoaded"
        }, {
            "name": "anjXAxisDidLoad",
            "method": "onXAxisLoaded"
        }, {
            "name": "anjYAxisDidLoad",
            "method": "onYAxisLoaded"
        }, {
            "name": "window:resize",
            "method": "handleResize",
            "passive": true
        }]; }
    static get style() { return "/**style-placeholder:anj-chart:**/"; }
}
Chart.creationDebounceLength = 50;
Chart.popoverDebounceLength = 150;
