import { h } from '../anjuna-charts.core.js';

import { a as d3 } from './chunk-1ad07b08.js';
import { a as ChartDataService, b as SeriesStyle } from './chunk-15621c8f.js';

var SeriesType;
(function (SeriesType) {
    SeriesType[SeriesType["Line"] = 0] = "Line";
    SeriesType[SeriesType["Column"] = 1] = "Column";
    SeriesType[SeriesType["Stacked"] = 2] = "Stacked";
})(SeriesType || (SeriesType = {}));

class ChartService {
    static addArea(svg, line, y, data, color) {
        const area = d3.area()
            .curve(d3.curveMonotoneX)
            .defined(line.defined())
            .x(line.x())
            .y1(line.y())
            .y0(y(0));
        svg.append('path')
            .data([data])
            .attr('class', 'series area')
            .attr('fill', color)
            .attr('d', area);
        return area;
    }
    static addStackedColumn(svg, x, y, data, color, _dims, _index) {
        const seriesContainers = svg
            .append('g')
            .attr('class', 'series-container')
            .selectAll('.series-container')
            .data([data])
            .enter().append('g')
            .attr('class', 'group')
            .style('fill', color);
        const chartBars = seriesContainers
            .selectAll('rect')
            .data((d) => d)
            .enter().append('rect');
        chartBars
            .attr('class', (d, i) => {
            return d[0] < 0 ? `bar negative bar-${i}` : `bar positive bar-${i}`;
        })
            .attr('width', x.bandwidth)
            .attr('x', (d) => x(d.data.categoryValue))
            .attr('y', y(0))
            .attr('height', 0);
        chartBars
            .transition()
            .duration(ChartService.animationLength)
            .attr('y', (d) => y(d[1]))
            .attr('height', (d) => y(d[0]) - y(d[1]));
        return chartBars;
    }
    static addColumn(svg, x, x1, y, data, color, _dims, index) {
        const seriesContainers = svg
            .append('g')
            .attr('class', 'series-container')
            .selectAll('.series-container')
            .data([data])
            .enter().append('g');
        const chartBars = seriesContainers
            .attr('class', 'series')
            .attr('transform', 'translate(' + x1(index) + ',0)')
            .style('fill', color)
            .selectAll('rect')
            .data((d) => d)
            .enter().append('rect');
        chartBars
            .attr('class', (d) => {
            return d.y < 0 ? 'bar negative' : 'bar positive';
        })
            .style('opacity', (d) => {
            return d.y < 0 ? .6 : 1;
        })
            .attr('width', x1.bandwidth())
            .attr('x', (d) => x(d.x))
            .attr('y', () => y(0))
            .attr('height', 0);
        chartBars
            .transition()
            .duration(ChartService.animationLength)
            .attr('y', (d) => d.y >= 0 ? (y(d.y)) : y(0))
            .attr('height', (d) => {
            return Math.max(Math.abs(y(d.y) - y(0)), 1);
        });
        return chartBars;
    }
    static addLine(svg, x, y, data, color) {
        const valueLine = d3.line()
            .curve(d3.curveMonotoneX)
            .defined((d) => d.y !== null)
            .x((d) => x(d.x))
            .y((d) => y(d.y));
        svg.append('path')
            .data([data])
            .attr('class', 'line')
            .attr('stroke', color)
            .attr('d', valueLine);
        return valueLine;
    }
    static addOverlay(svg, dims) {
        const overlay = svg.append('rect')
            .attr('class', 'overlay')
            .attr('width', dims.width)
            .attr('height', dims.height);
        return overlay;
    }
    static addPoints(svg, x, y, data, color, index) {
        svg.selectAll(`.point.series${index}`)
            .data(data)
            .enter()
            .append('svg:circle')
            .style('opacity', 0)
            .style('stroke', color)
            .attr('class', (_d, i) => `point point-${i}`)
            .attr('r', () => 1)
            .attr('cx', (d) => x(d.x))
            .attr('cy', (d) => y(d.y));
    }
    static addRevealAnimation(svg, dims, axisConfigs) {
        const delay = ChartService.focusLength;
        const duration = ChartService.animationLength;
        let curtainHeight = dims.height;
        let curtainWidth = dims.width;
        if (!axisConfigs.x.showAxis) {
            curtainHeight = curtainHeight + 1;
        }
        if (axisConfigs.y.showAxis) {
            curtainWidth = curtainWidth - (this.yPad * 2.5);
        }
        const curtain = svg.append('rect')
            .attr('x', -1 * dims.width)
            .attr('y', -1 * curtainHeight)
            .attr('height', curtainHeight)
            .attr('width', curtainWidth)
            .attr('class', 'curtain')
            .attr('transform', 'rotate(180)')
            .style('fill', '#ffffff');
        const t = svg.transition()
            .delay(delay)
            .duration(duration);
        t.select('rect.curtain')
            .attr('width', 0);
        t.select('line.guide')
            .attr('transform', 'translate(' + dims.width + ', 0)');
        return curtain;
    }
    static addStackedScrubbers(svg, x, data, dims, index) {
        const xOffset = ((x.bandwidth() + ChartService.calculatePadding(data, dims)) / 2);
        svg.selectAll(`.scrubber.series${index}`)
            .data(data)
            .enter()
            .append('line')
            .attr('x1', (d) => x(d.data.categoryValue) + xOffset)
            .attr('y1', 0)
            .attr('x2', (d) => x(d.data.categoryValue) + xOffset)
            .attr('y2', dims.height)
            .attr('class', (_d, i) => `scrubber scrubber-${i}`)
            .attr('stroke', '#ffffff')
            .style('opacity', '0');
    }
    static addScrubbers(svg, x, data, dims, index) {
        svg.selectAll(`.scrubber.series${index}`)
            .data(data)
            .enter()
            .append('line')
            .attr('x1', (d) => x(d.x))
            .attr('y1', 0)
            .attr('x2', (d) => x(d.x))
            .attr('y2', dims.height)
            .attr('class', (_d, i) => `scrubber scrubber-${i}`);
    }
    static getX1(x, data, multiSeries) {
        const seriesCount = (multiSeries) ? data.length : 1;
        return d3.scaleBand()
            .rangeRound([0, x.bandwidth()])
            .domain(d3.range(seriesCount))
            .padding(0);
    }
    static addXAxis(svg, dims, data, axisConfigs, _minMax, primarySeries) {
        const x = ChartService.axisTyper(primarySeries, data, dims)
            .range([this.calculateLeftPad(dims, axisConfigs), dims.width]);
        const xAxis = d3.axisBottom(x)
            .tickSizeOuter(0)
            .ticks(Math.min(this.calculateTicks(dims.width), data.length * 2))
            .tickFormat(ChartDataService.xFormatter.bind(this, ChartDataService.calculateInterval(data), axisConfigs.x))
            .scale(x);
        svg.append('g')
            .attr('class', 'x axis')
            .attr('transform', `translate(0,${dims.height})`)
            .call(xAxis);
        if (!axisConfigs.x.showAxis) {
            svg.select('.x.axis').style('display', 'none');
        }
        if (!axisConfigs.x.showTicks) {
            svg.selectAll('.x.axis .tick').style('display', 'none');
        }
        return x;
    }
    static axisTyper(type, data, dims) {
        switch (type) {
            case SeriesType.Line:
                return d3.scaleTime()
                    .domain(ChartDataService.determineDateRange(data));
            default:
                const domain = data.map((d) => d.x.toString());
                return d3.scaleBand()
                    .padding(ChartService.calculatePadding(data, dims))
                    .domain(domain);
        }
    }
    static calculatePadding(data, dims) {
        const colWidth = dims.width / (data.length - 1);
        return colWidth > 30 ? .05 : 0;
    }
    static addYAxis(svg, dims, data, axisConfigs, minMax) {
        const domain = (minMax) ? [minMax.min, minMax.max] : [0, d3.max(data, (d) => d.y)];
        const y = d3.scaleLinear()
            .domain(domain)
            .nice()
            .range([dims.height, dims.margin.top]);
        const yAxis = d3.axisLeft(y)
            .tickSize(0)
            .ticks(this.calculateTicks(dims.height))
            .tickFormat(ChartDataService.yFormatter.bind(this, axisConfigs.y))
            .scale(y);
        svg.append('g')
            .attr('class', 'y axis')
            .attr('transform', `translate(${this.calculateLeftPad(dims, axisConfigs)},0)`)
            .call(yAxis);
        if (!axisConfigs.y.showAxis) {
            svg.select('.y.axis').style('display', 'none');
        }
        if (!axisConfigs.y.showTicks) {
            svg.selectAll('.y.axis .tick').style('display', 'none');
        }
        return y;
    }
    static createSVG(dims) {
        const svg = d3.create('svg');
        svg
            .attr('width', dims.width + dims.margin.left)
            .attr('height', dims.height + dims.margin.top)
            .attr('transform', `translate(${dims.margin.left}, ${dims.margin.top})`)
            .append('g');
        return svg;
    }
    static getIndexOfClosestPoint(x, data) {
        const x0 = x.invert(d3.mouse(d3.event.currentTarget)[0]);
        const bisector = d3.bisector((d) => d.x).left;
        const index = bisector(data, x0, 1);
        const d0 = data[index - 1];
        const d1 = data[index];
        const rangeValueOfFirst = x((d0) ? d0.x : null), rangeValueOfSecond = x((d1) ? d1.x : null), rangeValueOfMousePos = d3.mouse(d3.event.currentTarget)[0], closestIndex = Math.abs(rangeValueOfMousePos - rangeValueOfFirst) > Math.abs(rangeValueOfMousePos - rangeValueOfSecond) ? index : index - 1;
        return closestIndex;
    }
    static hideFocus(svg) {
        svg.selectAll('.point')
            .transition()
            .duration(ChartService.focusLength)
            .attr('r', 1)
            .style('opacity', '0');
        svg.selectAll(`.scrubber`)
            .transition()
            .duration(ChartService.focusLength)
            .style('opacity', '0');
    }
    static showFocus(svg, activeIndex) {
        ChartService.hideFocus(svg);
        svg.selectAll(`.point-${activeIndex}`)
            .transition()
            .duration(ChartService.focusLength)
            .attr('r', 4)
            .style('opacity', '1');
        svg.selectAll(`.scrubber-${activeIndex}`)
            .transition()
            .duration(ChartService.focusLength)
            .style('opacity', '1');
    }
    static calculateLeftPad(dims, axisConfigs) {
        if (axisConfigs.y.showAxis || axisConfigs.y.showTicks) {
            return this.yPad + dims.margin.left;
        }
        else {
            return 0;
        }
    }
    static calculateTicks(dim) {
        return Math.max(dim / 100, 2);
    }
    static formatDataAsStacked(rawData) {
        const stackFactory = d3.stack()
            .offset(ChartService.stackOffsetDiverging)
            .keys(rawData.map((_s, i) => `namey-${i}`));
        const data = ChartService.seriesAsStackFormat(rawData);
        return stackFactory(data);
    }
    static seriesAsStackFormat(series) {
        const namedSeries = series.map((s, i) => {
            return {
                name: `namey-${i}`,
                coords: s
            };
        });
        const collector = [];
        namedSeries.forEach((s) => {
            s.coords.forEach((coords, i) => {
                if (!collector[i]) {
                    collector[i] = {};
                }
                collector[i].categoryValue = coords.x;
                collector[i][s.name] = coords.y;
            });
        });
        return collector;
    }
    static stackOffsetDiverging(series, seriesOrder) {
        const n = series.length;
        if (n < 1) {
            return;
        }
        const firstSeriesIndex = seriesOrder[0];
        for (let i, j = 0, dy, yp, yn, columnCount = series[firstSeriesIndex].length; j < columnCount; ++j) {
            for (yp = yn = 0, i = 0; i < n; ++i) {
                const serie = series[seriesOrder[i]];
                const dataPoint = serie[j];
                dy = dataPoint[1] - dataPoint[0];
                if (dy >= 0) {
                    dataPoint[0] = yp;
                    dataPoint[1] = yp += dy;
                }
                else if (dy < 0) {
                    dataPoint[1] = yn;
                    dataPoint[0] = yn += dy;
                }
                else {
                    dataPoint[0] = yp;
                }
            }
        }
    }
}
ChartService.colors = [
    '#384b5d',
    '#6ec5ab',
    '#6D95A0',
    '#995D81',
    '#586BA4',
    '#58A4B0',
    '#D496A7',
    '#C8AD55',
    '#8EA4D2',
    '#4ECDC4',
    '#98D2EB',
    '#857C8D'
];
ChartService.animationLength = 2500;
ChartService.focusLength = 400;
ChartService.yPad = 15;

class AxisEmitter {
    constructor(emitter) {
        this.emitter = emitter;
    }
    emit(axisConfig) {
        return this.emitter.emit(axisConfig);
    }
}
class AxisStrategy extends AxisEmitter {
}
class XAxisStrategy extends AxisStrategy {
    addAxis(svg, dims, data, axisConfigs, minMax, primarySeries) {
        return ChartService.addXAxis(svg, dims, data, axisConfigs, minMax, primarySeries);
    }
}
class YAxisStrategy extends AxisStrategy {
    addAxis(svg, dims, data, axisConfigs, minMax) {
        return ChartService.addYAxis(svg, dims, data, axisConfigs, minMax);
    }
}

class Axis {
    constructor() {
        this.dimension = 'x';
        this.show = true;
        this.ticks = true;
    }
    componentDidLoad() {
        this.service = (this.dimension === 'x') ? new XAxisStrategy(this.anjXAxisDidLoad) : new YAxisStrategy(this.anjYAxisDidLoad);
        const axisConfig = {
            showAxis: this.show,
            showTicks: this.ticks,
            formattingOptions: {
                locale: this.locale || 'en-US',
                language: this.language || 'en',
                currency: this.currency || 'USD'
            }
        };
        if (this.type) {
            axisConfig.labelType = this.type;
        }
        this.service.emit(axisConfig);
    }
    async init(svg, dims, data, axisConfigs, minMax, primarySeries) {
        return Promise.resolve(this.service.addAxis(svg, dims, data, axisConfigs, minMax, primarySeries));
    }
    static get is() { return "anj-axis"; }
    static get properties() { return {
        "currency": {
            "type": String,
            "attr": "currency"
        },
        "dimension": {
            "type": String,
            "attr": "dimension",
            "reflectToAttr": true
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        },
        "language": {
            "type": String,
            "attr": "language"
        },
        "locale": {
            "type": String,
            "attr": "locale"
        },
        "show": {
            "type": Boolean,
            "attr": "show"
        },
        "ticks": {
            "type": Boolean,
            "attr": "ticks"
        },
        "type": {
            "type": String,
            "attr": "type"
        }
    }; }
    static get events() { return [{
            "name": "anjYAxisDidLoad",
            "method": "anjYAxisDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "anjXAxisDidLoad",
            "method": "anjXAxisDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

class ElementStrategyFactory {
    static get(primaryType) {
        switch (primaryType) {
            case SeriesType.Line:
                return new LineStrategyFactory();
            case SeriesType.Stacked:
                return new StackedStrategyFactory();
            case SeriesType.Column:
            default:
                return new ColumnStrategyFactory();
        }
    }
}
class LineStrategyFactory {
    get(style) {
        switch (style) {
            case SeriesStyle.Multi:
                return new MultiLineSeries();
            case SeriesStyle.Single:
            default:
                return new SingleLineSeries();
        }
    }
}
class ColumnStrategyFactory {
    get(style) {
        switch (style) {
            case SeriesStyle.Multi:
                return new MultiColumnSeries();
            case SeriesStyle.Single:
            default:
                return new SingleColumnSeries();
        }
    }
}
class StackedStrategyFactory {
    get() {
        return new StackedSeries();
    }
}
class SingleSeriesChartStrategy {
    arrow() {
        return true;
    }
    flatten(data) {
        return data;
    }
    getPopoverEl(svg, index) {
        return svg.select('.point-' + index).nodes()[0];
    }
    getPopoverPlacement() {
        return 'top';
    }
    getSeriesStyle() {
        return SeriesStyle.Single;
    }
    getTargetingData(data) {
        return data;
    }
    isMultiSeries() {
        return false;
    }
}
class MultiSeriesChartStrategy {
    arrow() {
        return false;
    }
    flatten(data) {
        return [].concat(...data);
    }
    getPopoverEl(svg, index) {
        return svg.select('.scrubber-' + index).nodes()[0];
    }
    getPopoverPlacement() {
        return 'right';
    }
    getSeriesStyle() {
        return SeriesStyle.Multi;
    }
    getTargetingData(data) {
        return data[0];
    }
    isMultiSeries() {
        return true;
    }
}
class StackedSeries extends MultiSeriesChartStrategy {
}
class SingleLineSeries extends SingleSeriesChartStrategy {
}
class MultiLineSeries extends MultiSeriesChartStrategy {
}
class SingleColumnSeries extends SingleSeriesChartStrategy {
}
class MultiColumnSeries extends MultiSeriesChartStrategy {
    arrow() {
        return true;
    }
    getPopoverPlacement() {
        return 'top';
    }
}

class Chart {
    constructor() {
        this.service = new SingleLineSeries();
    }
    dataSourceHandler(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.debounceCreation();
        }
    }
    componentWillLoad() {
        this.axisConfigs = {
            x: this.getDefaultXAxisConfig(),
            y: this.getDefaultYAxisConfig()
        };
    }
    componentDidLoad() {
        this.debounceCreation();
    }
    async onColumnsLoaded() {
        this.columns = await this.loadColumns();
        this.debounceCreation();
    }
    async onLinesLoaded() {
        this.lines = await this.loadLines();
        this.debounceCreation();
    }
    async onStackedColumnsLoaded() {
        this.stackedColumns = await this.loadStackedColumns();
        this.debounceCreation();
    }
    async onXAxisLoaded({ detail }) {
        this.xAxisElement = await this.loadXAxis();
        this.axisConfigs.x = Object.assign({}, this.axisConfigs.x, detail);
        this.debounceCreation();
    }
    async onYAxisLoaded({ detail }) {
        this.yAxisElement = await this.loadYAxis();
        this.axisConfigs.y = Object.assign({}, this.axisConfigs.y, detail);
        this.debounceCreation();
    }
    handleResize() {
        this.debounceCreation();
    }
    anjHoverHandler(svg, data, x) {
        const targetData = this.service.getTargetingData(data);
        const dataIndex = ChartService.getIndexOfClosestPoint(x, targetData);
        let activeData;
        if (this.activeIndex !== dataIndex) {
            this.activeIndex = dataIndex;
            const seriesStyle = this.service.getSeriesStyle();
            activeData = { data, axisConfigs: this.axisConfigs, activeIndex: this.activeIndex, seriesStyle, seriesMeta: this.seriesMeta };
            this.anjHover.emit(activeData);
            this.popoverTimer = window.setTimeout(() => {
                const popoverEl = this.service.getPopoverEl(svg, this.activeIndex);
                this.display(popoverEl);
                ChartService.showFocus(svg, this.activeIndex);
            }, Chart.popoverDebounceLength);
        }
    }
    addLineEvents(svg, chartOverlay, x, data) {
        chartOverlay
            .on('mousemove', () => this.anjHoverHandler(svg, data, x))
            .on('mouseout', () => this.anjEndHoverHandler(svg));
    }
    addColumnEvents(svgBars, data) {
        svgBars
            .on('mousemove', (_d, i, bars) => this.anjColumnHoverHandler(bars[i], data, i))
            .on('mouseout', () => this.anjEndHoverHandler(svgBars));
    }
    anjEndHoverHandler(svg) {
        switch (this.primarySeries) {
            case SeriesType.Line:
                window.clearTimeout(this.popoverTimer);
                this.activeIndex = null;
                this.popover.close();
                ChartService.hideFocus(svg);
            default:
                this.popoverTimer = window.setTimeout(() => {
                    this.activeIndex = null;
                    this.popover.close();
                    ChartService.hideFocus(svg);
                }, Chart.popoverDebounceLength);
        }
    }
    anjColumnHoverHandler(targetEl, data, index) {
        window.clearTimeout(this.popoverTimer);
        let activeData;
        activeData = { data, axisConfigs: this.axisConfigs, activeIndex: index, seriesStyle: SeriesStyle.Single, seriesMeta: this.seriesMeta };
        this.anjHover.emit(activeData);
        this.popoverEl = targetEl;
        this.display(this.popoverEl);
    }
    anjStackedHoverHandler(svg, data, dataIndex) {
        window.clearTimeout(this.popoverTimer);
        let activeData;
        if (this.activeIndex !== dataIndex) {
            this.activeIndex = dataIndex;
            const seriesStyle = this.service.getSeriesStyle();
            activeData = { data, axisConfigs: this.axisConfigs, activeIndex: this.activeIndex, seriesStyle, seriesMeta: this.seriesMeta };
            this.anjHover.emit(activeData);
            this.popoverEl = this.service.getPopoverEl(svg, this.activeIndex);
            this.display(this.popoverEl);
            ChartService.showFocus(svg, this.activeIndex);
        }
    }
    addStackedEvents(svg, bars) {
        bars
            .on('mousemove', (_d, i) => this.anjStackedHoverHandler(svg, this.data, i))
            .on('mouseout', () => this.anjEndHoverHandler(svg));
    }
    appendSVG(svg) {
        while (this.chartContainer.children.length > 1) {
            this.chartContainer.removeChild(this.chartContainer.lastChild);
        }
        this.chartContainer.appendChild(svg.node());
    }
    debounceCreation() {
        window.clearTimeout(this.creationTimer);
        this.creationTimer = window.setTimeout(async () => {
            this.chartSVG = await this.create();
            console.log(this.chartSVG);
        }, Chart.creationDebounceLength);
    }
    async display(target) {
        const popper = await this.popover.popper();
        if (popper) {
            popper.reference = target;
            popper.scheduleUpdate();
        }
        else {
            this.popover.open(target);
        }
    }
    async create() {
        let minMax;
        let stackedData;
        this.primarySeries = (this.stackedColumns) ? SeriesType.Stacked : (this.lines && !this.columns) ? SeriesType.Line : SeriesType.Column;
        this.serviceFactory = ElementStrategyFactory.get(this.primarySeries);
        if (this.primarySeries === SeriesType.Line) {
            this.axisConfigs.x.labelType = 'date';
        }
        this.data = this.handleDataSource();
        const dims = this.getDims();
        const flattenedData = this.service.flatten(this.data);
        if (this.stackedColumns) {
            stackedData = ChartService.formatDataAsStacked(this.data);
            const flattenedPointValues = stackedData.flat().flat();
            minMax = { min: Math.min(...flattenedPointValues), max: Math.max(...flattenedPointValues) };
            console.log('>>', stackedData);
        }
        const svg = ChartService.createSVG(dims);
        const x = (this.xAxisElement)
            ? await this.hydrateXAxis(dims, flattenedData, svg, this.primarySeries)
            : ChartService.addXAxis(svg, dims, flattenedData, this.axisConfigs, undefined, this.primarySeries);
        const y = (this.yAxisElement)
            ? await this.hydrateYAxis(dims, flattenedData, svg, minMax)
            : ChartService.addYAxis(svg, dims, flattenedData, this.axisConfigs, minMax);
        if (this.columns) {
            const x1 = ChartService.getX1(x, this.data, this.multiSeries);
            await this.hydrateColumns(svg, this.data, x, x1, y, dims);
        }
        if (this.stackedColumns) {
            await this.hydrateStackedColumns(svg, stackedData, x, y, dims);
        }
        if (this.lines) {
            await this.hydrateLines(svg, this.data, x, y, dims);
            if (this.primarySeries === SeriesType.Line) {
                const chartOverlay = ChartService.addOverlay(svg, dims);
                this.addLineEvents(svg, chartOverlay, x, this.data);
                ChartService.addRevealAnimation(svg, dims, this.axisConfigs);
            }
        }
        this.appendSVG(svg);
        return Promise.resolve(svg);
    }
    getDefaultXAxisConfig() {
        return {
            showAxis: false,
            labelType: 'string',
        };
    }
    getDefaultYAxisConfig() {
        return {
            showAxis: false,
            labelType: 'number',
        };
    }
    getDims() {
        const margin = { top: 20, right: 20, bottom: 20, left: 20 };
        const { height, width } = this.chartContainer.getBoundingClientRect();
        return {
            height: height - margin.top - margin.bottom,
            width: width - margin.left - margin.right,
            margin
        };
    }
    handleDataSource() {
        let formattedData;
        if (Array.isArray(this.dataSource[0])) {
            this.service = this.serviceFactory.get(SeriesStyle.Multi);
            formattedData = this.dataSource.map(d => ChartDataService.formatData(this.axisConfigs, d));
        }
        else {
            this.service = this.serviceFactory.get(SeriesStyle.Single);
            formattedData = ChartDataService.formatData(this.axisConfigs, this.dataSource);
        }
        this.multiSeries = this.service.isMultiSeries();
        this.popover.placement = this.service.getPopoverPlacement();
        return formattedData;
    }
    async hydrateColumns(svg, data, x, x1, y, dims) {
        this.seriesMeta = [];
        this.columns.forEach(async (column, i) => {
            const index = (!isNaN(column.dataIndex)) ? column.dataIndex : i;
            const columnData = (this.service.isMultiSeries()) ? data[index] : data;
            const bars = await column.init(svg, x, x1, y, columnData, index, dims);
            this.seriesMeta[index] = { name: column.name, netNegative: column.netNegative, color: undefined };
            this.addColumnEvents(bars, columnData);
        });
    }
    async hydrateLines(svg, data, x, y, dims) {
        this.seriesMeta = [];
        this.lines.forEach(async (line, i) => {
            const index = (!isNaN(line.dataIndex)) ? line.dataIndex : i;
            const lineData = (this.service.isMultiSeries()) ? data[index] : data;
            const color = await line.init(svg, x, y, lineData, index, dims, this.service.isMultiSeries());
            this.seriesMeta[index] = { name: line.name, color, netNegative: line.netNegative };
        });
    }
    async hydrateStackedColumns(svg, data, x, y, dims) {
        this.seriesMeta = [];
        this.stackedColumns.forEach(async (column, i) => {
            const index = (!isNaN(column.dataIndex)) ? column.dataIndex : i;
            const columnData = (this.service.isMultiSeries()) ? data[index] : data;
            const { bars, color } = await column.init(svg, x, y, columnData, index, dims);
            this.seriesMeta[index] = { name: column.name, netNegative: column.netNegative, color };
            this.addStackedEvents(svg, bars);
        });
    }
    hydrateXAxis(dims, data, svg, primarySeries) {
        return this.xAxisElement.init(svg, dims, data, this.axisConfigs, undefined, primarySeries);
    }
    hydrateYAxis(dims, data, svg, minMax) {
        return this.yAxisElement.init(svg, dims, data, this.axisConfigs, minMax);
    }
    loadColumns() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-column-series')).map(col => col.componentOnReady()));
    }
    loadLines() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-line-series')).map(col => col.componentOnReady()));
    }
    loadStackedColumns() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-stacked-column-series')).map(col => col.componentOnReady()));
    }
    loadXAxis() {
        return Promise.resolve(this.chartContainer.querySelector('anj-axis[dimension="x"]').componentOnReady());
    }
    loadYAxis() {
        return Promise.resolve(this.chartContainer.querySelector('anj-axis[dimension="y"]').componentOnReady());
    }
    hostData() {
        return {
            class: {
                'anj-rm-pop-arrow': !this.service.arrow()
            }
        };
    }
    render() {
        return [
            h("anj-overlay", { class: "popover", placement: "top", offset: "0, 15", ref: r => this.popover = r },
                h("anj-arrow", null),
                h("slot", null))
        ];
    }
    static get is() { return "anj-chart"; }
    static get properties() { return {
        "chartContainer": {
            "elementRef": true
        },
        "dataSource": {
            "type": "Any",
            "attr": "data-source",
            "mutable": true,
            "watchCallbacks": ["dataSourceHandler"]
        },
        "multiSeries": {
            "state": true
        }
    }; }
    static get events() { return [{
            "name": "anjHover",
            "method": "anjHover",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get listeners() { return [{
            "name": "anjColumnDidLoad",
            "method": "onColumnsLoaded"
        }, {
            "name": "anjLineDidLoad",
            "method": "onLinesLoaded"
        }, {
            "name": "anjStackedColumnDidLoad",
            "method": "onStackedColumnsLoaded"
        }, {
            "name": "anjXAxisDidLoad",
            "method": "onXAxisLoaded"
        }, {
            "name": "anjYAxisDidLoad",
            "method": "onYAxisLoaded"
        }, {
            "name": "window:resize",
            "method": "handleResize",
            "passive": true
        }]; }
    static get style() { return "anj-chart{display:block;height:100%;position:relative;width:100%}anj-chart .overlay{cursor:default;fill:none;pointer-events:all;position:relative;z-index:3}anj-chart svg .area{opacity:.1}anj-chart svg .axis{font-size:.5rem}anj-chart svg .axis line,anj-chart svg .axis path{fill:none;stroke:var(--anj-border);shape-rendering:crispEdges}anj-chart svg .axis.y path{stroke:transparent}anj-chart svg .tick text,anj-chart svg text{fill:var(--anj-neutral-light)}anj-chart svg .line{fill:none;stroke-width:2px;-webkit-transition:opacity .5s ease,stroke-width .5s ease;transition:opacity .5s ease,stroke-width .5s ease}anj-chart svg .overlay{fill:none;pointer-events:all;position:relative;z-index:3}anj-chart svg .point{fill:var(--anj-light);stroke-width:1.5px}anj-chart.anj-rm-pop-arrow anj-arrow{display:none}anj-chart .popover{pointer-events:none}anj-chart .series-container .bar{opacity:1;-webkit-transition:opacity .5s;transition:opacity .5s}"; }
}
Chart.creationDebounceLength = 50;
Chart.popoverDebounceLength = 150;

class ColumnSeries {
    componentDidLoad() {
        this.anjColumnDidLoad.emit();
    }
    async init(svg, x, x1, y, data, index, dims) {
        this.color = this.color || ChartService.colors[index || 0];
        const bars = ChartService.addColumn(svg, x, x1, y, data, this.color, dims, index);
        return Promise.resolve(bars);
    }
    static get is() { return "anj-column-series"; }
    static get properties() { return {
        "color": {
            "type": String,
            "attr": "color",
            "mutable": true
        },
        "dataIndex": {
            "type": Number,
            "attr": "data-index"
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        },
        "name": {
            "type": String,
            "attr": "name"
        },
        "netNegative": {
            "type": Boolean,
            "attr": "net-negative"
        }
    }; }
    static get events() { return [{
            "name": "anjColumnDidLoad",
            "method": "anjColumnDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

class LineSeries {
    componentDidLoad() {
        this.anjLineDidLoad.emit();
    }
    async init(svg, x, y, data, index, dims, multiSeries) {
        this.color = this.color || ChartService.colors[index || 0];
        const line = ChartService.addLine(svg, x, y, data, this.color);
        if (multiSeries) {
            ChartService.addScrubbers(svg, x, data, dims, index);
        }
        else {
            ChartService.addArea(svg, line, y, data, this.color);
        }
        ChartService.addPoints(svg, x, y, data, this.color, index);
        return Promise.resolve(this.color);
    }
    static get is() { return "anj-line-series"; }
    static get properties() { return {
        "color": {
            "type": String,
            "attr": "color",
            "mutable": true
        },
        "dataIndex": {
            "type": Number,
            "attr": "data-index"
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        },
        "name": {
            "type": String,
            "attr": "name"
        },
        "netNegative": {
            "type": Boolean,
            "attr": "net-negative"
        }
    }; }
    static get events() { return [{
            "name": "anjLineDidLoad",
            "method": "anjLineDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

class StackedColumnSeries {
    componentDidLoad() {
        this.anjStackedColumnDidLoad.emit();
    }
    async init(svg, x, y, data, index, dims) {
        this.color = this.color || ChartService.colors[index || 0];
        const bars = ChartService.addStackedColumn(svg, x, y, data, this.color, dims, index);
        ChartService.addStackedScrubbers(svg, x, data, dims, index);
        return Promise.resolve({ bars, color: this.color });
    }
    static get is() { return "anj-stacked-column-series"; }
    static get properties() { return {
        "color": {
            "type": String,
            "attr": "color",
            "mutable": true
        },
        "dataIndex": {
            "type": Number,
            "attr": "data-index"
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        },
        "name": {
            "type": String,
            "attr": "name"
        },
        "netNegative": {
            "type": Boolean,
            "attr": "net-negative"
        }
    }; }
    static get events() { return [{
            "name": "anjStackedColumnDidLoad",
            "method": "anjStackedColumnDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

export { Axis as AnjAxis, Chart as AnjChart, ColumnSeries as AnjColumnSeries, LineSeries as AnjLineSeries, StackedColumnSeries as AnjStackedColumnSeries };
